/*
 * Copyright 2012 Red Hat Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Author: Jerome Glisse
 */
#include "atom.h"

#if ATOM_DEBUG

static char tmp_str[32];

/*
 * ATOM_IO_NAMES
 */
#define ATOM_IO_NAMES_CNT 5
static const char *atom_io_names[ATOM_IO_NAMES_CNT] = {
	"MM", "PLL", "MC", "PCIE", "PCIE PORT",
};

const char *atom_io_name(unsigned io)
{
	if (io < ATOM_IO_NAMES_CNT) {
		return atom_io_names[io];
	}
	return "ATOM_IO_UNKNOWN";
}


/*
 * ATOM_OP_NAMES
 */
#define ATOM_OP_NAMES_CNT 123
static const char *atom_op_names[ATOM_OP_NAMES_CNT] = {
	"RESERVED", "MOVE_REG", "MOVE_PS", "MOVE_WS", "MOVE_FB", "MOVE_PLL",
	"MOVE_MC", "AND_REG", "AND_PS", "AND_WS", "AND_FB", "AND_PLL", "AND_MC",
	"OR_REG", "OR_PS", "OR_WS", "OR_FB", "OR_PLL", "OR_MC", "SHIFT_LEFT_REG",
	"SHIFT_LEFT_PS", "SHIFT_LEFT_WS", "SHIFT_LEFT_FB", "SHIFT_LEFT_PLL",
	"SHIFT_LEFT_MC", "SHIFT_RIGHT_REG", "SHIFT_RIGHT_PS", "SHIFT_RIGHT_WS",
	"SHIFT_RIGHT_FB", "SHIFT_RIGHT_PLL", "SHIFT_RIGHT_MC", "MUL_REG",
	"MUL_PS", "MUL_WS", "MUL_FB", "MUL_PLL", "MUL_MC", "DIV_REG", "DIV_PS",
	"DIV_WS", "DIV_FB", "DIV_PLL", "DIV_MC", "ADD_REG", "ADD_PS", "ADD_WS",
	"ADD_FB", "ADD_PLL", "ADD_MC", "SUB_REG", "SUB_PS", "SUB_WS", "SUB_FB",
	"SUB_PLL", "SUB_MC", "SET_ATI_PORT", "SET_PCI_PORT", "SET_SYS_IO_PORT",
	"SET_REG_BLOCK", "SET_FB_BASE", "COMPARE_REG", "COMPARE_PS",
	"COMPARE_WS", "COMPARE_FB", "COMPARE_PLL", "COMPARE_MC", "SWITCH",
	"JUMP", "JUMP_EQUAL", "JUMP_BELOW", "JUMP_ABOVE", "JUMP_BELOW_OR_EQUAL",
	"JUMP_ABOVE_OR_EQUAL", "JUMP_NOT_EQUAL", "TEST_REG", "TEST_PS", "TEST_WS",
	"TEST_FB", "TEST_PLL", "TEST_MC", "DELAY_MILLISEC", "DELAY_MICROSEC",
	"CALL_TABLE", "REPEAT", "CLEAR_REG", "CLEAR_PS", "CLEAR_WS", "CLEAR_FB",
	"CLEAR_PLL", "CLEAR_MC", "NOP", "EOT", "MASK_REG", "MASK_PS", "MASK_WS",
	"MASK_FB", "MASK_PLL", "MASK_MC", "POST_CARD", "BEEP", "SAVE_REG",
	"RESTORE_REG", "SET_DATA_BLOCK", "XOR_REG", "XOR_PS", "XOR_WS", "XOR_FB",
	"XOR_PLL", "XOR_MC", "SHL_REG", "SHL_PS", "SHL_WS", "SHL_FB", "SHL_PLL",
	"SHL_MC", "SHR_REG", "SHR_PS", "SHR_WS", "SHR_FB", "SHR_PLL", "SHR_MC",
	"DEBUG", "CTB_DS",
};

const char *atom_op_name(unsigned op)
{
	if (op < ATOM_OP_NAMES_CNT) {
		return atom_op_names[op];
	}
	return "ATOM_OP_UNKNOWN";
}


/*
 * ATOM_TABLE_NAMES
 */
#define ATOM_TABLE_NAMES_CNT 81
static const char *atom_table_names[ATOM_TABLE_NAMES_CNT] = {
	"ASIC_Init",				"GetDisplaySurfaceSize",
	"ASIC_RegistersInit",			"VRAM_BlockVenderDetection",
	"DIGxEncoderControl",			"MemoryControllerInit",
	"EnableCRTCMemReq",			"MemoryParamAdjust",
	"DVOEncoderControl",			"GPIOPinControl",
	"SetEngineClock",			"SetMemoryClock",
	"SetPixelClock",			"DynamicClockGating",
	"ResetMemoryDLL",			"ResetMemoryDevice",
	"MemoryPLLInit",			"AdjustDisplayPll",
	"AdjustMemoryController",		"EnableASIC_StaticPwrMgt",
	"ASIC_StaticPwrMgtStatusChange",	"DAC_LoadDetection",
	"TMDS2EncoderControl",			"LCD1OutputControl",
	"DAC1EncoderControl",			"DAC2EncoderControl",
	"DVOOutputControl",			"CV1OutputControl",
	"SetCRTC_DPM_State",			"TVEncoderControl",
	"TMDS1EncoderControl",			"LVDSEncoderControl",
	"TV1OutputControl",			"EnableScaler",
	"BlankCRTC",				"EnableCRTC",
	"GetPixelClock",			"EnableVGA_Render",
	"EnableVGA_Access",			"SetCRTC_Timing",
	"SetCRTC_OverScan",			"SetCRTC_Replication",
	"SelectCRTC_Source",			"EnableGraphSurfaces",
	"UpdateCRTC_DoubleBufferRegisters",	"LUT_AutoFill",
	"EnableHW_IconCursor",			"GetMemoryClock",
	"GetEngineClock",			"SetCRTC_UsingDTDTiming",
	"ExternalEncoderControl",		"DFP2OutputControl",
	"VRAM_BlockDetectionByStrap",		"MemoryCleanUp",
	"ReadEDIDFromHWAssistedI2C",		"WriteOneByteToHWAssistedI2C",
	"ReadHWAssistedI2CStatus",		"SpeedFanControl",
	"PowerConnectorDetection",		"MC_Synchronization",
	"ComputeMemoryEnginePLL",		"MemoryRefreshConversion",
	"VRAM_GetCurrentInfoBlock",		"DynamicMemorySettings",
	"MemoryTraining",			"EnableLVDS_SS",
	"DFP1OutputControl",			"SetVoltage",
	"CRT1OutputControl",			"CRT2OutputControl",
	"SetupHWAssistedI2CStatus",		"ClockSource",
	"MemoryDeviceInit",			"EnableYUV",
	"DIG1EncoderControl",			"DIG2EncoderControl",
	"DIG1TransmitterControl",		"DIG2TransmitterControl",
	"ProcessAuxChannelTransaction",		"DPEncoderService",
	"GetVoltageInfo",
};

const char *atom_command_table_name(unsigned index)
{
	if (index < ATOM_TABLE_NAMES_CNT) {
		return atom_table_names[index];
	}
	return "COMMAND_TABLE_UNKNOWN";
}


/*
 * CONNECTOR_OBJECT_ID
 */
#define	CONNECTOR_OBJECT_ID_CNT				23
static const char *atom_connector_object_id[CONNECTOR_OBJECT_ID_CNT] = {
	"CONNECTOR_OBJECT_ID_NONE",
	"CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_I",
	"CONNECTOR_OBJECT_ID_DUAL_LINK_DVI_I",
	"CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_D",
	"CONNECTOR_OBJECT_ID_DUAL_LINK_DVI_D",
	"CONNECTOR_OBJECT_ID_VGA",
	"CONNECTOR_OBJECT_ID_COMPOSITE",
	"CONNECTOR_OBJECT_ID_SVIDEO",
	"CONNECTOR_OBJECT_ID_YPbPr",
	"CONNECTOR_OBJECT_ID_D_CONNECTOR",
	"CONNECTOR_OBJECT_ID_9PIN_DIN",
	"CONNECTOR_OBJECT_ID_SCART",
	"CONNECTOR_OBJECT_ID_HDMI_TYPE_A",
	"CONNECTOR_OBJECT_ID_HDMI_TYPE_B",
	"CONNECTOR_OBJECT_ID_LVDS",
	"CONNECTOR_OBJECT_ID_7PIN_DIN",
	"CONNECTOR_OBJECT_ID_PCIE_CONNECTOR",
	"CONNECTOR_OBJECT_ID_CROSSFIRE",
	"CONNECTOR_OBJECT_ID_HARDCODE_DVI",
	"CONNECTOR_OBJECT_ID_DISPLAYPORT",
	"CONNECTOR_OBJECT_ID_eDP",
	"CONNECTOR_OBJECT_ID_MXM",
	"CONNECTOR_OBJECT_ID_LVDS_eDP",
};

static const char *p_atom_connector_object_id(unsigned v)
{
	if (v >= CONNECTOR_OBJECT_ID_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return atom_connector_object_id[v];
}


/*
 * ENCODER_OBJECT_ID
 */
#define ENCODER_OBJECT_ID_CNT				37
static const char *atom_encoder_object_id[ENCODER_OBJECT_ID_CNT] = {
	"ENCODER_OBJECT_ID_NONE",
	"ENCODER_OBJECT_ID_INTERNAL_LVDS",
	"ENCODER_OBJECT_ID_INTERNAL_TMDS1",
	"ENCODER_OBJECT_ID_INTERNAL_TMDS2",
	"ENCODER_OBJECT_ID_INTERNAL_DAC1",
	"ENCODER_OBJECT_ID_INTERNAL_DAC2",
	"ENCODER_OBJECT_ID_INTERNAL_SDVOA",
	"ENCODER_OBJECT_ID_INTERNAL_SDVOB",
	"ENCODER_OBJECT_ID_SI170B",
	"ENCODER_OBJECT_ID_CH7303",
	"ENCODER_OBJECT_ID_CH7301",
	"ENCODER_OBJECT_ID_INTERNAL_DVO1",
	"ENCODER_OBJECT_ID_EXTERNAL_SDVOA",
	"ENCODER_OBJECT_ID_EXTERNAL_SDVOB",
	"ENCODER_OBJECT_ID_TITFP513",
	"ENCODER_OBJECT_ID_INTERNAL_LVTM1",
	"ENCODER_OBJECT_ID_VT1623",
	"ENCODER_OBJECT_ID_HDMI_SI1930",
	"ENCODER_OBJECT_ID_HDMI_INTERNAL",
	"ENCODER_OBJECT_ID_INTERNAL_KLDSCP_TMDS1",
	"ENCODER_OBJECT_ID_INTERNAL_KLDSCP_DVO1",
	"ENCODER_OBJECT_ID_INTERNAL_KLDSCP_DAC1",
	"ENCODER_OBJECT_ID_INTERNAL_KLDSCP_DAC2",
	"ENCODER_OBJECT_ID_SI178",
	"ENCODER_OBJECT_ID_MVPU_FPGA",
	"ENCODER_OBJECT_ID_INTERNAL_DDI",
	"ENCODER_OBJECT_ID_VT1625",
	"ENCODER_OBJECT_ID_HDMI_SI1932",
	"ENCODER_OBJECT_ID_DP_AN9801",
	"ENCODER_OBJECT_ID_DP_DP501",
	"ENCODER_OBJECT_ID_INTERNAL_UNIPHY",
	"ENCODER_OBJECT_ID_INTERNAL_KLDSCP_LVTMA",
	"ENCODER_OBJECT_ID_INTERNAL_UNIPHY1",
	"ENCODER_OBJECT_ID_INTERNAL_UNIPHY2",
	"ENCODER_OBJECT_ID_NUTMEG",
	"ENCODER_OBJECT_ID_TRAVIS",
	"ENCODER_OBJECT_ID_INTERNAL_VCE",
};

static const char *p_atom_encoder_object_id(unsigned v)
{
	if (v >= ENCODER_OBJECT_ID_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return atom_encoder_object_id[v];
}

/*
 * ENCODER_ID
 */
#define ATOM_ENCODER_ID_CNT				15
static const char *atom_encoder_id[ATOM_ENCODER_ID_CNT] = {
	"ASIC_INT_DAC1_ENCODER_ID",
	"ASIC_ENCODER_ID_UNKNOWN_1",
	"ASIC_INT_TV_ENCODER_ID",
	"ASIC_INT_DIG1_ENCODER_ID",
	"ASIC_INT_DAC2_ENCODER_ID",
	"ASIC_EXT_DIG_ENCODER_ID",
	"ASIC_EXT_TV_ENCODER_ID",
	"ASIC_INT_DVO_ENCODER_ID",
	"ASIC_EXT_DIG2_ENCODER_ID",
	"ASIC_INT_DIG2_ENCODER_ID",
	"ASIC_INT_DIG3_ENCODER_ID",
	"ASIC_INT_DIG4_ENCODER_ID",
	"ASIC_INT_DIG5_ENCODER_ID",
	"ASIC_INT_DIG6_ENCODER_ID",
	"ASIC_INT_DIG7_ENCODER_ID",
};

static const char *p_atom_encoder_id(unsigned v)
{
	if (v >= ATOM_ENCODER_ID_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return atom_encoder_id[v];
}

/*
 * ATOM_ENCODER_MODE
 */
#define ATOM_ENCODER_MODE_CNT				17
static const char *atom_encoder_mode[ATOM_ENCODER_MODE_CNT] = {
	"ATOM_ENCODER_MODE_DP",
	"ATOM_ENCODER_MODE_LVDS",
	"ATOM_ENCODER_MODE_DVI",
	"ATOM_ENCODER_MODE_HDMI",
	"ATOM_ENCODER_MODE_SDVO",
	"ATOM_ENCODER_MODE_DP_AUDIO",
	"ATOM_ENCODER_MODE_UNKNOWN_6",
	"ATOM_ENCODER_MODE_UNKNOWN_7",
	"ATOM_ENCODER_MODE_UNKNOWN_8",
	"ATOM_ENCODER_MODE_UNKNOWN_9",
	"ATOM_ENCODER_MODE_UNKNOWN_10",
	"ATOM_ENCODER_MODE_UNKNOWN_11",
	"ATOM_ENCODER_MODE_UNKNOWN_12",
	"ATOM_ENCODER_MODE_TV",
	"ATOM_ENCODER_MODE_CV",
	"ATOM_ENCODER_MODE_CRT",
	"ATOM_ENCODER_MODE_DVO",
};

static const char *p_atom_encoder_mode(unsigned v)
{
	if (v >= ATOM_ENCODER_MODE_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return atom_encoder_mode[v];
}


/*
 * ATOM_TRANSMITTER_ACTION
 */
#define	ATOM_TRANSMITTER_ACTION_CNT			14
static const char *atom_transmitter_action[ATOM_TRANSMITTER_ACTION_CNT] = {
	"ATOM_TRANSMITTER_ACTION_DISABLE",
	"ATOM_TRANSMITTER_ACTION_ENABLE",
	"ATOM_TRANSMITTER_ACTION_LCD_BLOFF",
	"ATOM_TRANSMITTER_ACTION_LCD_BLON",
	"ATOM_TRANSMITTER_ACTION_BL_BRIGHTNESS_CONTROL",
	"ATOM_TRANSMITTER_ACTION_LCD_SELFTEST_START",
	"ATOM_TRANSMITTER_ACTION_LCD_SELFTEST_STOP",
	"ATOM_TRANSMITTER_ACTION_INIT",
	"ATOM_TRANSMITTER_ACTION_DISABLE_OUTPUT",
	"ATOM_TRANSMITTER_ACTION_ENABLE_OUTPUT",
	"ATOM_TRANSMITTER_ACTION_SETUP",
	"ATOM_TRANSMITTER_ACTION_SETUP_VSEMPH",
	"ATOM_TRANSMITTER_ACTION_POWER_ON",
	"ATOM_TRANSMITTER_ACTION_POWER_OFF",
};

static const char *p_atom_transmitter_action(unsigned v)
{
	if (v >= ATOM_TRANSMITTER_ACTION_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return atom_transmitter_action[v];
}


/*
 * CONNECTOR_OBJECT_ID
 */
#define	CONNECTOR_OBJECT_ID_CNT				23
static const char *connector_object_id[CONNECTOR_OBJECT_ID_CNT] = {
	"CONNECTOR_OBJECT_ID_NONE",
	"CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_I",
	"CONNECTOR_OBJECT_ID_DUAL_LINK_DVI_I",
	"CONNECTOR_OBJECT_ID_SINGLE_LINK_DVI_D",
	"CONNECTOR_OBJECT_ID_DUAL_LINK_DVI_D",
	"CONNECTOR_OBJECT_ID_VGA",
	"CONNECTOR_OBJECT_ID_COMPOSITE",
	"CONNECTOR_OBJECT_ID_SVIDEO",
	"CONNECTOR_OBJECT_ID_YPbPr",
	"CONNECTOR_OBJECT_ID_D_CONNECTOR",
	"CONNECTOR_OBJECT_ID_9PIN_DIN",
	"CONNECTOR_OBJECT_ID_SCART",
	"CONNECTOR_OBJECT_ID_HDMI_TYPE_A",
	"CONNECTOR_OBJECT_ID_HDMI_TYPE_B",
	"CONNECTOR_OBJECT_ID_LVDS",
	"CONNECTOR_OBJECT_ID_7PIN_DIN",
	"CONNECTOR_OBJECT_ID_PCIE_CONNECTOR",
	"CONNECTOR_OBJECT_ID_CROSSFIRE",
	"CONNECTOR_OBJECT_ID_HARDCODE_DVI",
	"CONNECTOR_OBJECT_ID_DISPLAYPORT",
	"CONNECTOR_OBJECT_ID_eDP",
	"CONNECTOR_OBJECT_ID_MXM",
	"CONNECTOR_OBJECT_ID_LVDS_eDP",
};

static const char *p_connector_object_id(unsigned v)
{
	if (v >= CONNECTOR_OBJECT_ID_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return connector_object_id[v];
}


/*
 * EXTERNAL_ENCODER_ACTION_V3
 */
#define	EXTERNAL_ENCODER_ACTION_V3_CNT			21
static const char *external_encoder_action_v3[EXTERNAL_ENCODER_ACTION_V3_CNT] = {
	"EXTERNAL_ENCODER_ACTION_V3_DISABLE_OUTPUT",
	"EXTERNAL_ENCODER_ACTION_V3_ENABLE_OUTPUT",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_2",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_3",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_4",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_5",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_6",
	"EXTERNAL_ENCODER_ACTION_V3_ENCODER_INIT",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_8",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_9",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_10",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_11",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_12",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_13",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_14",
	"EXTERNAL_ENCODER_ACTION_V3_ENCODER_SETUP",
	"EXTERNAL_ENCODER_ACTION_V3_ENCODER_BLANKING_OFF",
	"EXTERNAL_ENCODER_ACTION_V3_ENCODER_BLANKING",
	"EXTERNAL_ENCODER_ACTION_V3_DACLOAD_DETECTION",
	"EXTERNAL_ENCODER_ACTION_V3_UNKNOWN_19",
	"EXTERNAL_ENCODER_ACTION_V3_DDC_SETUP",
};

static const char *p_external_encoder_action_v3(unsigned v)
{
	if (v >= EXTERNAL_ENCODER_ACTION_V3_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return external_encoder_action_v3[v];
}

/*
 * ENCODER_ACTION_V2
 */
#define	ENCODER_ACTION_V2_CNT			20
static const char *encoder_action_v2[ENCODER_ACTION_V2_CNT] = {
	"ATOM_ENCODER_CMD_UNKNOWN_0",
	"ATOM_ENCODER_CMD_UNKNOWN_1",
	"ATOM_ENCODER_CMD_UNKNOWN_2",
	"ATOM_ENCODER_CMD_UNKNOWN_3",
	"ATOM_ENCODER_CMD_UNKNOWN_4",
	"ATOM_ENCODER_CMD_UNKNOWN_5",
	"ATOM_ENCODER_CMD_UNKNOWN_6",
	"ATOM_ENCODER_CMD_UNKNOWN_7",
	"ATOM_ENCODER_CMD_DP_LINK_TRAINING_START",
	"ATOM_ENCODER_CMD_DP_LINK_TRAINING_PATTERN1",
	"ATOM_ENCODER_CMD_DP_LINK_TRAINING_PATTERN2",
	"ATOM_ENCODER_CMD_DP_LINK_TRAINING_COMPLETE",
	"ATOM_ENCODER_CMD_DP_VIDEO_OFF",
	"ATOM_ENCODER_CMD_DP_VIDEO_ON",
	"ATOM_ENCODER_CMD_QUERY_DP_LINK_TRAINING_STATUS",
	"ATOM_ENCODER_CMD_SETUP",
	"ATOM_ENCODER_CMD_SETUP_PANEL_MODE",
	"ATOM_ENCODER_CMD_UNKNOWN_17",
	"ATOM_ENCODER_CMD_UNKNOWN_18",
	"ATOM_ENCODER_CMD_DP_LINK_TRAINING_PATTERN3",
};

static const char *p_encoder_action_v2(unsigned v)
{
	if (v >= ENCODER_ACTION_V2_CNT) {
		sprintf(tmp_str, "%d", v);
		return tmp_str;
	}
	return encoder_action_v2[v];
}


/*
 * ATOM_MASTER_LIST_OF_COMMAND_TABLES
 */
typedef void (*atom_table_print_args_t)(const char *prefix, void *args);

struct atom_table_trace {
	unsigned		index;
	unsigned		frev;
	unsigned		crev;
	atom_table_print_args_t print;
	const char		*name;
};

static void p_dig_transmitter_config_v3(const char *prefix, void *args);
static void p_dig_transmitter_control_parameters_v3(const char *prefix, void *args);
static void p_enable_crtc_parameters(const char *prefix, void *args);
static void p_pixel_clock_parameters_v5(const char *prefix, void *args);
static void p_set_crtc_overscan(const char *prefix, void *args);
static void p_select_crtc_source_v2(const char *prefix, void *args);
static void p_adjust_display_pll_v3(const char *prefix, void *args);
static void p_enable_lvds_ss_v2(const char *prefix, void *args);
static void p_set_crtc_using_dtd_timing(const char *prefix, void *args);
static void p_compute_memory_engine_pll_v4(const char *prefix, void *args);
static void p_blank_crtc(const char *prefix, void *args);
static void p_enable_crtc_mem_req(const char *prefix, void *args);
static void p_external_encoder_control_v3(const char *prefix, void *args);
static void p_digx_encoder_control_v2(const char *prefix, void *args);
static void p_process_aux_channel_transaction_v2(const char *prefix, void *args);
static void p_get_pixel_clock_v2(const char *prefix, void *args);
static void p_gpio_pin_control_v1(const char *prefix, void *args);
static void p_enable_scaler_v1(const char *prefix, void *args);

static const struct atom_table_trace atom_table_trace[] = {
	{ 4,  1,  2,  p_digx_encoder_control_v2, "DIGxEncoderControl"},
	{ 6,  2,  1,  p_enable_crtc_mem_req, "EnableCRTCMemReq"},
	{ 9,  2,  1,  p_gpio_pin_control_v1, "GPIOPinControl"},
	{12,  1,  5,  p_pixel_clock_parameters_v5, "SetPixelClock"},
	{17,  1,  3,  p_adjust_display_pll_v3, "AdjustDisplayPll"},
	{33,  1,  1,  p_enable_scaler_v1, "EnableScaler"},
	{34,  1,  1,  p_blank_crtc, "BlankCRTC"},
	{35,  1,  1,  p_enable_crtc_parameters, "EnableCRTC"},
	{36,  1,  2,  p_get_pixel_clock_v2, "GetPixelClock"},
	{40,  1,  1,  p_set_crtc_overscan, "SetCRTC_OverScan"},
	{42,  1,  2,  p_select_crtc_source_v2, "SelectCRTC_Source"},
	{44,  1,  1,  p_enable_crtc_parameters, "UpdateCRTC_DoubleBufferRegisters"},
	{49,  2,  3,  p_set_crtc_using_dtd_timing, "SetCRTC_UsingDTDTiming"},
	{50,  2,  3,  p_external_encoder_control_v3, "ExternalEncoderControl"},
	{60,  1,  4,  p_compute_memory_engine_pll_v4, "ComputeMemoryEnginePLL"},
	{65,  1,  2,  p_enable_lvds_ss_v2, "EnableLVDS_SS"},
	{76,  1,  3,  p_dig_transmitter_control_parameters_v3, "DIG1TransmitterControl"},
//	{78,  1,  2,  p_process_aux_channel_transaction_v2, "ProcessAuxChannelTransaction"},
};

static void p_dig_transmitter_config_v3(const char *prefix, void *args)
{
	ATOM_DIG_TRANSMITTER_CONFIG_V3 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tATOM_DIG_TRANSMITTER_CONFIG_V3 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.fDualLinkConnector = %d;\n", prefix, pargs->fDualLinkConnector);
	printk(KERN_INFO "%s.fCoherentMode = %d;\n", prefix, pargs->fCoherentMode);
	printk(KERN_INFO "%s.ucLinkSel = %d;\n", prefix, pargs->ucLinkSel);
	printk(KERN_INFO "%s.ucEncoderSel = %d;\n", prefix, pargs->ucEncoderSel);
	printk(KERN_INFO "%s.ucRefClkSource = %d;\n", prefix, pargs->ucRefClkSource);
	printk(KERN_INFO "%s.ucTransmitterSel = %d;\n", prefix, pargs->ucTransmitterSel);
}

static void p_dig_transmitter_control_parameters_v3(const char *prefix, void *args)
{
	DIG_TRANSMITTER_CONTROL_PARAMETERS_V3 *pargs = args;
	char tmp[128];

	if (prefix == NULL) {
		printk(KERN_INFO "\tDIG_TRANSMITTER_CONTROL_PARAMETERS_V3 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.usInitInfo = %s;\n", prefix,
		p_atom_connector_object_id(pargs->usInitInfo));
	printk(KERN_INFO "%s.asMode.ucLaneSel = %d;\n", prefix, pargs->asMode.ucLaneSel);
	printk(KERN_INFO "%s.asMode.ucLaneSet = %d;\n", prefix, pargs->asMode.ucLaneSet);
	sprintf(tmp, "%s.%s", prefix, "acConfig");
	p_dig_transmitter_config_v3(tmp, &pargs->acConfig);
	printk(KERN_INFO "%s.ucAction = %s;\n", prefix,
		p_atom_transmitter_action(pargs->ucAction));
	printk(KERN_INFO "%s.ucLaneNum = %d;\n", prefix, pargs->ucLaneNum);
}

static void p_enable_crtc_parameters(const char *prefix, void *args)
{
	ENABLE_CRTC_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tENABLE_CRTC_PARAMETERS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	printk(KERN_INFO "%s.ucEnable = %d;\n", prefix, pargs->ucEnable);
}

static void p_pixel_clock_parameters_v5(const char *prefix, void *args)
{
	PIXEL_CLOCK_PARAMETERS_V5 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tPIXEL_CLOCK_PARAMETERS_V5 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	printk(KERN_INFO "%s.ucReserved = %d;\n", prefix, pargs->ucReserved);
	printk(KERN_INFO "%s.ucFracFbDiv = %d;\n", prefix, pargs->ucFracFbDiv);
	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.usFbDiv = %d;\n", prefix, pargs->usFbDiv);
	printk(KERN_INFO "%s.ucPostDiv = %d;\n", prefix, pargs->ucPostDiv);
	printk(KERN_INFO "%s.ucRefDiv = %d;\n", prefix, pargs->ucRefDiv);
	printk(KERN_INFO "%s.ucRefDiv = %d;\n", prefix, pargs->ucRefDiv);
	printk(KERN_INFO "%s.ucPpll = %d;\n", prefix, pargs->ucPpll);
	printk(KERN_INFO "%s.ucTransmitterID = %s;\n", prefix,
		p_atom_encoder_object_id(pargs->ucTransmitterID));
	printk(KERN_INFO "%s.ucEncoderMode = %d;\n", prefix, pargs->ucEncoderMode);
	printk(KERN_INFO "%s.ucMiscInfo = 0x%x;\n", prefix, pargs->ucMiscInfo);
	printk(KERN_INFO "%s.ulFbDivDecFrac = %d;\n", prefix, pargs->ulFbDivDecFrac);
}

static void p_set_crtc_overscan(const char *prefix, void *args)
{
	SET_CRTC_OVERSCAN_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tSET_CRTC_OVERSCAN_PARAMETERS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usOverscanRight = %d;\n", prefix, pargs->usOverscanRight);
	printk(KERN_INFO "%s.usOverscanLeft = %d;\n", prefix, pargs->usOverscanLeft);
	printk(KERN_INFO "%s.usOverscanBottom = %d;\n", prefix, pargs->usOverscanBottom);
	printk(KERN_INFO "%s.usOverscanTop = %d;\n", prefix, pargs->usOverscanTop);
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
}

static void p_select_crtc_source_v2(const char *prefix, void *args)
{
	SELECT_CRTC_SOURCE_PARAMETERS_V2 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tSELECT_CRTC_SOURCE_PARAMETERS_V2 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	printk(KERN_INFO "%s.ucEncoderID = %s;\n", prefix,
		p_atom_encoder_id(pargs->ucEncoderID));
	printk(KERN_INFO "%s.ucEncodeMode = %s;\n", prefix,
		p_atom_encoder_mode(pargs->ucEncodeMode));
}

static void p_adjust_display_pll_v3(const char *prefix, void *args)
{
	ADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tADJUST_DISPLAY_PLL_INPUT_PARAMETERS_V3 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.ucTransmitterID = %s;\n", prefix,
		p_atom_encoder_object_id(pargs->ucTransmitterID));
	printk(KERN_INFO "%s.ucEncodeMode = %s;\n", prefix,
		p_atom_encoder_mode(pargs->ucEncodeMode));
	printk(KERN_INFO "%s.ucDispPllConfig = 0x%x;\n", prefix, pargs->ucDispPllConfig);
	printk(KERN_INFO "%s.ucExtTransmitterID = %s;\n", prefix,
		p_atom_encoder_object_id(pargs->ucTransmitterID));
}

static void p_enable_lvds_ss_v2(const char *prefix, void *args)
{
	ENABLE_LVDS_SS_PARAMETERS_V2 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tENABLE_LVDS_SS_PARAMETERS_V2 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usSpreadSpectrumPercentage = %d;\n", prefix, pargs->usSpreadSpectrumPercentage);
	printk(KERN_INFO "%s.ucSpreadSpectrumType = 0x%x;\n", prefix, pargs->ucSpreadSpectrumType);
	printk(KERN_INFO "%s.ucSpreadSpectrumStep = %d;\n", prefix, pargs->ucSpreadSpectrumStep);
	printk(KERN_INFO "%s.ucEnable = %d;\n", prefix, pargs->ucEnable);
	printk(KERN_INFO "%s.ucSpreadSpectrumDelay = %d;\n", prefix, pargs->ucSpreadSpectrumDelay);
	printk(KERN_INFO "%s.ucSpreadSpectrumRange = %d;\n", prefix, pargs->ucSpreadSpectrumRange);
}

static void p_atom_misc_info_access(const char *prefix, void *args)
{
	ATOM_MODE_MISC_INFO_ACCESS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tATOM_MODE_MISC_INFO_ACCESS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.sbfAccess.HorizontalCutOff = %d;\n", prefix, pargs->sbfAccess.HorizontalCutOff);
	printk(KERN_INFO "%s.sbfAccess.HSyncPolarity = %d;\n", prefix, pargs->sbfAccess.HSyncPolarity);
	printk(KERN_INFO "%s.sbfAccess.VSyncPolarity = %d;\n", prefix, pargs->sbfAccess.VSyncPolarity);
	printk(KERN_INFO "%s.sbfAccess.VerticalCutOff = %d;\n", prefix, pargs->sbfAccess.VerticalCutOff);
	printk(KERN_INFO "%s.sbfAccess.H_ReplicationBy2 = %d;\n", prefix, pargs->sbfAccess.H_ReplicationBy2);
	printk(KERN_INFO "%s.sbfAccess.V_ReplicationBy2 = %d;\n", prefix, pargs->sbfAccess.V_ReplicationBy2);
	printk(KERN_INFO "%s.sbfAccess.CompositeSync = %d;\n", prefix, pargs->sbfAccess.CompositeSync);
	printk(KERN_INFO "%s.sbfAccess.Interlace = %d;\n", prefix, pargs->sbfAccess.Interlace);
	printk(KERN_INFO "%s.sbfAccess.DoubleClock = %d;\n", prefix, pargs->sbfAccess.DoubleClock);
	printk(KERN_INFO "%s.sbfAccess.RGB888 = %d;\n", prefix, pargs->sbfAccess.RGB888);
	printk(KERN_INFO "%s.usAccess = 0x%x;\n", prefix, pargs->usAccess);
}

static void p_set_crtc_using_dtd_timing(const char *prefix, void *args)
{
	SET_CRTC_USING_DTD_TIMING_PARAMETERS *pargs = args;
	char tmp[128];

	if (prefix == NULL) {
		printk(KERN_INFO "\tSET_CRTC_USING_DTD_TIMING_PARAMETERS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usH_Size = %d;\n", prefix, pargs->usH_Size);
	printk(KERN_INFO "%s.usH_Blanking_Time = %d;\n", prefix, pargs->usH_Blanking_Time);
	printk(KERN_INFO "%s.usV_Size = %d;\n", prefix, pargs->usV_Size);
	printk(KERN_INFO "%s.usV_Blanking_Time = %d;\n", prefix, pargs->usV_Blanking_Time);
	printk(KERN_INFO "%s.usH_SyncOffset = %d;\n", prefix, pargs->usH_SyncOffset);
	printk(KERN_INFO "%s.usH_SyncWidth = %d;\n", prefix, pargs->usH_SyncWidth);
	printk(KERN_INFO "%s.usV_SyncOffset = %d;\n", prefix, pargs->usV_SyncOffset);
	printk(KERN_INFO "%s.usV_SyncWidth = %d;\n", prefix, pargs->usV_SyncWidth);
	sprintf(tmp, "%s.%s", prefix, "susModeMiscInfo");
	p_atom_misc_info_access(tmp, &pargs->susModeMiscInfo);
	printk(KERN_INFO "%s.ucH_Border = %d;\n", prefix, pargs->ucH_Border);
	printk(KERN_INFO "%s.ucV_Border = %d;\n", prefix, pargs->ucV_Border);
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
}

static void p_compute_memory_engine_pll_v4(const char *prefix, void *args)
{
	COMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tCOMPUTE_MEMORY_ENGINE_PLL_PARAMETERS_V4 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ulClock = %d;\n", prefix, pargs->ulClock);
	printk(KERN_INFO "%s.ucPostDiv = %d;\n", prefix, pargs->ucPostDiv);
}

static void p_blank_crtc(const char *prefix, void *args)
{
	BLANK_CRTC_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tBLANK_CRTC_PARAMETERS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	switch (pargs->ucBlanking) {
	case ATOM_BLANKING:
		printk(KERN_INFO "%s.ucBlanking = ATOM_BLANKING;\n", prefix);
		break;
	case ATOM_BLANKING_OFF:
		printk(KERN_INFO "%s.ucBlanking = ATOM_BLANKING_OFF;\n", prefix);
		break;
	default:
		printk(KERN_INFO "%s.ucBlanking = %d;\n", prefix, pargs->ucBlanking);
		break;
	}
	printk(KERN_INFO "%s.usBlackColorRCr = %d;\n", prefix, pargs->usBlackColorRCr);
	printk(KERN_INFO "%s.usBlackColorGY = %d;\n", prefix, pargs->usBlackColorGY);
	printk(KERN_INFO "%s.usBlackColorBCb = %d;\n", prefix, pargs->usBlackColorBCb);
}

static void p_enable_crtc_mem_req(const char *prefix, void *args)
{
	ENABLE_CRTC_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tENABLE_CRTC_PARAMETERS args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	printk(KERN_INFO "%s.ucEnable = %d;\n", prefix, pargs->ucEnable);
}

static void p_external_encoder_control_v3(const char *prefix, void *args)
{
	EXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tEXTERNAL_ENCODER_CONTROL_PARAMETERS_V3 args;\n");
		prefix = "\targs";
	};
	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.usConnectorId = %s;\n", prefix,
		p_connector_object_id(pargs->usConnectorId));
	printk(KERN_INFO "%s.ucConfig = 0x%x;\n", prefix, pargs->ucConfig);
	printk(KERN_INFO "%s.ucAction = %s;\n", prefix,
		p_external_encoder_action_v3(pargs->ucAction));
	printk(KERN_INFO "%s.ucEncoderMode = %s;\n", prefix,
		p_atom_encoder_mode(pargs->ucEncoderMode));
	printk(KERN_INFO "%s.ucLaneNum = %d;\n", prefix, pargs->ucLaneNum);
	printk(KERN_INFO "%s.ucBitPerColor = %d;\n", prefix, pargs->ucBitPerColor);
}

// DIGxEncoderControl
static void p_digx_encoder_control_v2(const char *prefix, void *args)
{
	DIG_ENCODER_CONTROL_PARAMETERS_V2 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tDIG_ENCODER_CONTROL_PARAMETERS_V2 args;\n");
		prefix = "\targs";
	};

	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.acConfig.ucDPLinkRate = %d;\n", prefix, pargs->acConfig.ucDPLinkRate);
	printk(KERN_INFO "%s.acConfig.ucLinkSel = %d;\n", prefix, pargs->acConfig.ucLinkSel);
	printk(KERN_INFO "%s.acConfig.ucTransmitterSel = %d;\n", prefix, pargs->acConfig.ucTransmitterSel);
	printk(KERN_INFO "%s.ucAction = %s;\n", prefix, p_encoder_action_v2(pargs->ucAction));
	printk(KERN_INFO "%s.ucEncoderMode = %s;\n", prefix, p_atom_encoder_mode(pargs->ucEncoderMode));
	printk(KERN_INFO "%s.ucLaneNum = %d;\n", prefix, pargs->ucLaneNum);
	printk(KERN_INFO "%s.ucStatus = %d;\n", prefix, pargs->ucStatus);
}

// ProcessAuxChannelTransaction
static void p_process_aux_channel_transaction_v2(const char *prefix, void *args)
{
	PROCESS_AUX_CHANNEL_TRANSACTION_PARAMETERS_V2 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tPROCESS_AUX_CHANNEL_TRANSACTION_PARAMETERS_V2 args;\n");
		prefix = "\targs";
	};

	printk(KERN_INFO "%s.lpAuxRequest = 0x%x;\n", prefix, pargs->lpAuxRequest);
	printk(KERN_INFO "%s.lpDataOut = 0x%x;\n", prefix, pargs->lpDataOut);
	printk(KERN_INFO "%s.ucChannelID = %d;\n", prefix, pargs->ucChannelID);
	printk(KERN_INFO "%s.ucReplyStatus = %d;\n", prefix, pargs->ucReplyStatus);
	printk(KERN_INFO "%s.ucDelay = %d;\n", prefix, pargs->ucDelay);
	printk(KERN_INFO "%s.ucDataOutLen = %d;\n", prefix, pargs->ucDataOutLen);
	printk(KERN_INFO "%s.ucHPD_ID = %d;\n", prefix, pargs->ucHPD_ID);
}

// GetPixelClock
static void p_get_pixel_clock_v2(const char *prefix, void *args)
{
	PIXEL_CLOCK_PARAMETERS_V2 *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tPIXEL_CLOCK_PARAMETERS_V2 args;\n");
		prefix = "\targs";
	};

	printk(KERN_INFO "%s.usPixelClock = %d;\n", prefix, pargs->usPixelClock);
	printk(KERN_INFO "%s.usRefDiv = %d;\n", prefix, pargs->usRefDiv);
	printk(KERN_INFO "%s.usFbDiv = %d;\n", prefix, pargs->usFbDiv);
	printk(KERN_INFO "%s.ucPostDiv = %d;\n", prefix, pargs->ucPostDiv);
	printk(KERN_INFO "%s.ucFracFbDiv = %d;\n", prefix, pargs->ucFracFbDiv);
	printk(KERN_INFO "%s.ucPpll = %d;\n", prefix, pargs->ucPpll);
	printk(KERN_INFO "%s.ucRefDivSrc = %d;\n", prefix, pargs->ucRefDivSrc);
	printk(KERN_INFO "%s.ucCRTC = %d;\n", prefix, pargs->ucCRTC);
	printk(KERN_INFO "%s.ucMiscInfo = 0x%x;\n", prefix, pargs->ucMiscInfo);
}

// GPIOPinControl
static void p_gpio_pin_control_v1(const char *prefix, void *args)
{
	GPIO_PIN_CONTROL_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tGPIO_PIN_CONTROL_PARAMETERS args;\n");
		prefix = "\targs";
	};

	printk(KERN_INFO "%s.ucGPIO_ID = %d;\n", prefix, pargs->ucGPIO_ID);
	printk(KERN_INFO "%s.ucGPIOBitShift = %d;\n", prefix, pargs->ucGPIOBitShift);
	printk(KERN_INFO "%s.ucGPIOBitVal = %d;\n", prefix, pargs->ucGPIOBitVal);
	printk(KERN_INFO "%s.ucAction = %d;\n", prefix, pargs->ucAction);
}

// EnableScaler
static void p_enable_scaler_v1(const char *prefix, void *args)
{
	ENABLE_SCALER_PARAMETERS *pargs = args;

	if (prefix == NULL) {
		printk(KERN_INFO "\tENABLE_SCALER_PARAMETERS args;\n");
		prefix = "\targs";
	};

	printk(KERN_INFO "%s.ucScaler = %d;\n", prefix, pargs->ucScaler);
	printk(KERN_INFO "%s.ucEnable = %d;\n", prefix, pargs->ucEnable);
	printk(KERN_INFO "%s.ucTVStandard = %d;\n", prefix, pargs->ucTVStandard);
}

/*
 * tracer
 */
void atom_trace_cmd_header(struct atom_context *ctx, unsigned index, void *arg)
{
	unsigned i;
	uint8_t frev, crev;

	if (!atom_parse_cmd_header(ctx, index, &frev, &crev)) {
		printk(KERN_INFO "// Unsupported %d frev ?? crev ??\n", index);
		return;
	}

	for (i = 0; i < sizeof(atom_table_trace)/sizeof(atom_table_trace[0]); i++) {
		if (atom_table_trace[i].index == index &&
		    atom_table_trace[i].frev == frev &&
		    atom_table_trace[i].crev == crev) {
			printk(KERN_INFO "{\n");
			printk(KERN_INFO "\t// %s (%d) frev %d crev %d\n",
				atom_table_names[index], index, frev, crev);
			atom_table_trace[i].print(NULL, arg);
			printk(KERN_INFO "\tatom_execute_table(atom_ctx, %d, (uint32_t*)&args);\n", index);
			printk(KERN_INFO "}\n");
			return;
		}
	}
	if (index < ATOM_TABLE_NAMES_CNT) {
		printk(KERN_INFO "// Unsupported %s (%d) frev %d crev %d\n",
			atom_table_names[index], index, frev, crev);
	} else {
		printk(KERN_INFO "// Unsupported %d frev %d crev %d\n", index, frev, crev);
	}
}

#endif /* ATOM_DEBUG */
