/*
 * Copyright 2010 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * on the rights to use, copy, modify, merge, publish, distribute, sub
 * license, and/or sell copies of the Software, and to permit persons to whom
 * the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Authors:
 *      Jerome Glisse
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "radeon_pci.h"
#include "reg.h"

int main(int argc, char *argv[])
{
    struct dump tdump[4] = {{0, },};
    struct dump adump = {0, };
    struct dump bdump = {0, };
    struct dump ddump = {0, };
    unsigned i, j;

    if (dump_read(&tdump[0], argv[1])) {
        PERR("error reading [1] %s\n", argv[1]);
        return -1;
    }
    for (i = 2, j = 0; i < argc; i++) {
        if (!strcmp(argv[i], "-")) {
            adump = tdump[j + 0];
            tdump[j + 0].nregs = 0;
            tdump[j + 0].regs = NULL;
            j = 0;
            i++;
            if (dump_read(&tdump[j + 0], argv[i])) {
                PERR("error reading [%d] %s\n", i, argv[i]);
                return -1;
            }
            continue;
        }
        if (dump_read(&tdump[j + 2], argv[i])) {
            PERR("error reading [%d] %s\n", i, argv[i]);
            return -1;
        }
        if (dump_keep_same(&tdump[j + 1], &tdump[j + 0], &tdump[j + 2])) {
            PERR("error processing [%d] %s\n", i, argv[i]);
            return -1;
        }
        dump_fini(&tdump[j + 0]);
        dump_fini(&tdump[j + 2]);
        j++;
        j &= 3;
    }
    bdump = tdump[j + 0];
    if (dump_keep_diff(&ddump, &bdump, &adump)) {
        PERR("error diffing\n");
        return -1;
    }
    dump_print(&ddump);
    return 0;
}
