/*
 * Copyright 2010 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * on the rights to use, copy, modify, merge, publish, distribute, sub
 * license, and/or sell copies of the Software, and to permit persons to whom
 * the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Authors:
 *      Jerome Glisse
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "xf86drm.h"
#include "radeon_drm.h"
#include "radeon.h"

unsigned radeon_family_from_device(unsigned device);

static int radeon_get_device(struct radeon *radeon)
{
    struct drm_radeon_info info;
    int r;

    radeon->device = 0;
    info.request = RADEON_INFO_DEVICE_ID;
    info.value = (uintptr_t)&radeon->device;
    r = drmCommandWriteRead(radeon->fd, DRM_RADEON_INFO, &info,
                            sizeof(struct drm_radeon_info));
    return r;
}

int radeon_init(struct radeon *radeon)
{
    int fd;
    int r;

    fd = drmOpen("radeon", NULL);
    if (fd < 0) {
        PERR("failed opening radeon drm file\n");
        return -errno;
    }

    radeon->fd = fd;
    radeon->device = 0;
    r = radeon_get_device(radeon);
    if (r) {
        PERR("Failed to get device id\n");
        radeon_fini(radeon);
        return -EINVAL;
    }
    radeon->family = radeon_family_from_device(radeon->device);
    if (radeon->family == CHIP_UNKNOWN) {
        PERR("Unknown chipset 0x%04X\n", radeon->device);
        return -EINVAL;
    }
    return 0;
}

void radeon_fini(struct radeon *radeon)
{
    drmClose(radeon->fd);
}
